@@ -3,7 +3,12 @@
 // R-tree implementation
 //
 // Copyright (c) 2008 Federico J. Fernandez.
-// Copyright (c) 2011-2017 Adam Wulkiewicz, Lodz, Poland.
+// Copyright (c) 2011-2022 Adam Wulkiewicz, Lodz, Poland.
+// Copyright (c) 2020 Caian Benedicto, Campinas, Brazil.
+//
+// This file was modified by Oracle on 2019-2021.
+// Modifications copyright (c) 2019-2021 Oracle and/or its affiliates.
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 //
 // Use, modification and distribution is subject to the Boost Software License,
 // Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
@@ -14,12 +19,16 @@
 
 // STD
 #include <algorithm>
+#include <type_traits>
 
 // Boost
-#include <boost/tuple/tuple.hpp>
+#include <boost/container/new_allocator.hpp>
 #include <boost/move/move.hpp>
+#include <boost/tuple/tuple.hpp>
 
 // Boost.Geometry
+#include <boost/geometry/core/static_assert.hpp>
+
 #include <boost/geometry/algorithms/detail/comparable_distance/interface.hpp>
 #include <boost/geometry/algorithms/detail/covered_by/interface.hpp>
 #include <boost/geometry/algorithms/detail/disjoint/interface.hpp>
@@ -81,10 +90,21 @@
 #include <boost/geometry/index/detail/rtree/query_iterators.hpp>
 
 #ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
-// serialization
+#ifndef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
+#define BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
+#endif
+#ifndef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_ITERATORS
+#define BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_ITERATORS
+#endif
+#endif
+
+#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
 #include <boost/geometry/index/detail/serialization.hpp>
 #endif
 
+#include <boost/geometry/util/range.hpp>
+#include <boost/geometry/util/type_traits.hpp>
+
 // TODO change the name to bounding_tree
 
 /*!
@@ -106,7 +126,7 @@ algorithm with specific parameters like min and max number of elements in node.
 
 \par
 Predefined algorithms with compile-time parameters are:
-\li <tt>boost::geometry::index::linear</tt>,
+ \li <tt>boost::geometry::index::linear</tt>,
  \li <tt>boost::geometry::index::quadratic</tt>,
  \li <tt>boost::geometry::index::rstar</tt>.
 
@@ -123,8 +143,8 @@ access. Therefore the IndexableGetter should return the Indexable by
 a reference type. The Indexable should not be calculated since it could harm
 the performance. The default IndexableGetter can translate all types adapted
 to Point, Box or Segment concepts (called Indexables). Furthermore, it can
-handle <tt>std::pair<Indexable, T></tt>, <tt>boost::tuple<Indexable, ...></tt>
-and <tt>std::tuple<Indexable, ...></tt> when possible. For example, for Value
+handle <tt>std::pair<Indexable, T></tt>, <tt>std::tuple<Indexable, ...></tt>
+and <tt>boost::tuple<Indexable, ...></tt>. For example, for Value
 of type <tt>std::pair<Box, int></tt>, the default IndexableGetter translates
 from <tt>std::pair<Box, int> const&</tt> to <tt>Box const&</tt>.
 
@@ -149,7 +169,7 @@ template
     typename Parameters,
     typename IndexableGetter = index::indexable<Value>,
     typename EqualTo = index::equal_to<Value>,
-    typename Allocator = std::allocator<Value>
+    typename Allocator = boost::container::new_allocator<Value>
 >
 class rtree
 {
@@ -185,23 +205,122 @@ public:
 
 private:
 
-    typedef detail::translator<IndexableGetter, EqualTo> translator_type;
-
     typedef bounds_type box_type;
-    typedef typename detail::rtree::options_type<Parameters>::type options_type;
-    typedef typename options_type::node_tag node_tag;
-    typedef detail::rtree::allocators<allocator_type, value_type, typename options_type::parameters_type, box_type, node_tag> allocators_type;
 
-    typedef typename detail::rtree::node<value_type, typename options_type::parameters_type, box_type, allocators_type, node_tag>::type node;
-    typedef typename detail::rtree::internal_node<value_type, typename options_type::parameters_type, box_type, allocators_type, node_tag>::type internal_node;
-    typedef typename detail::rtree::leaf<value_type, typename options_type::parameters_type, box_type, allocators_type, node_tag>::type leaf;
+    struct members_holder
+        : public detail::translator<IndexableGetter, EqualTo>
+        , public Parameters
+        , public detail::rtree::allocators
+            <
+                Allocator,
+                Value,
+                Parameters,
+                bounds_type,
+                typename detail::rtree::options_type<Parameters>::type::node_tag
+            >
+    {
+        typedef Value value_type;
+        typedef typename rtree::bounds_type bounds_type;
+        typedef Parameters parameters_type;
+        //typedef IndexableGetter indexable_getter;
+        //typedef EqualTo value_equal;
+        //typedef Allocator allocator_type;
 
-    typedef typename allocators_type::node_pointer node_pointer;
-    typedef ::boost::container::allocator_traits<Allocator> allocator_traits_type;
-    typedef detail::rtree::subtree_destroyer<value_type, options_type, translator_type, box_type, allocators_type> subtree_destroyer;
+        typedef bounds_type box_type;
+        typedef detail::translator<IndexableGetter, EqualTo> translator_type;
+        typedef typename detail::rtree::options_type<Parameters>::type options_type;
+        typedef typename options_type::node_tag node_tag;
+        typedef detail::rtree::allocators
+            <
+                Allocator, Value, Parameters, bounds_type, node_tag
+            > allocators_type;
+
+        typedef typename detail::rtree::node
+            <
+                value_type, parameters_type, bounds_type, allocators_type, node_tag
+            >::type node;
+        typedef typename detail::rtree::internal_node
+            <
+                value_type, parameters_type, bounds_type, allocators_type, node_tag
+            >::type internal_node;
+        typedef typename detail::rtree::leaf
+            <
+                value_type, parameters_type, bounds_type, allocators_type, node_tag
+            >::type leaf;
+
+        // TODO: only one visitor type is needed
+        typedef typename detail::rtree::visitor
+            <
+                value_type, parameters_type, bounds_type, allocators_type, node_tag, false
+            >::type visitor;
+        typedef typename detail::rtree::visitor
+            <
+                value_type, parameters_type, bounds_type, allocators_type, node_tag, true
+            >::type visitor_const;
+
+        typedef typename allocators_type::node_pointer node_pointer;
+
+        typedef ::boost::container::allocator_traits<Allocator> allocator_traits_type;
+        typedef typename allocators_type::size_type size_type;
+
+    private:
+        members_holder(members_holder const&);
+        members_holder & operator=(members_holder const&);
+
+    public:
+        template <typename IndGet, typename ValEq, typename Alloc>
+        members_holder(IndGet const& ind_get,
+                       ValEq const& val_eq,
+                       Parameters const& parameters,
+                       BOOST_FWD_REF(Alloc) alloc)
+            : translator_type(ind_get, val_eq)
+            , Parameters(parameters)
+            , allocators_type(boost::forward<Alloc>(alloc))
+            , values_count(0)
+            , leafs_level(0)
+            , root(0)
+        {}
+
+        template <typename IndGet, typename ValEq>
+        members_holder(IndGet const& ind_get,
+                       ValEq const& val_eq,
+                       Parameters const& parameters)
+            : translator_type(ind_get, val_eq)
+            , Parameters(parameters)
+            , allocators_type()
+            , values_count(0)
+            , leafs_level(0)
+            , root(0)
+        {}
+
+        translator_type const& translator() const { return *this; }
+
+        IndexableGetter const& indexable_getter() const { return *this; }
+        IndexableGetter & indexable_getter() { return *this; }
+        EqualTo const& equal_to() const { return *this; }
+        EqualTo & equal_to() { return *this; }
+        Parameters const& parameters() const { return *this; }
+        Parameters & parameters() { return *this; }
+        allocators_type const& allocators() const { return *this; }
+        allocators_type & allocators() { return *this; }
+
+        size_type values_count;
+        size_type leafs_level;
+        node_pointer root;
+    };
+
+    typedef typename members_holder::translator_type translator_type;    
+    typedef typename members_holder::options_type options_type;
+    typedef typename members_holder::allocators_type allocators_type;
+    typedef typename members_holder::node node;
+    typedef typename members_holder::internal_node internal_node;
+    typedef typename members_holder::leaf leaf;
+
+    typedef typename members_holder::node_pointer node_pointer;
+    typedef typename members_holder::allocator_traits_type allocator_traits_type;
 
     friend class detail::rtree::utilities::view<rtree>;
-#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
+#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_SERIALIZATION
     friend class detail::rtree::private_view<rtree>;
     friend class detail::rtree::const_private_view<rtree>;
 #endif
@@ -294,12 +413,7 @@ public:
                  allocator_type const& allocator = allocator_type())
         : m_members(getter, equal, parameters, allocator)
     {
-        typedef detail::rtree::pack<value_type, options_type, translator_type, box_type, allocators_type> pack;
-        size_type vc = 0, ll = 0;
-        m_members.root = pack::apply(first, last, vc, ll,
-                                     m_members.parameters(), m_members.translator(), m_members.allocators());
-        m_members.values_count = vc;
-        m_members.leafs_level = ll;
+        pack_construct(first, last, boost::container::new_allocator<void>());
     }
 
     /*!
@@ -326,12 +440,156 @@ public:
                           allocator_type const& allocator = allocator_type())
         : m_members(getter, equal, parameters, allocator)
     {
-        typedef detail::rtree::pack<value_type, options_type, translator_type, box_type, allocators_type> pack;
-        size_type vc = 0, ll = 0;
-        m_members.root = pack::apply(::boost::begin(rng), ::boost::end(rng), vc, ll,
-                                     m_members.parameters(), m_members.translator(), m_members.allocators());
-        m_members.values_count = vc;
-        m_members.leafs_level = ll;
+        pack_construct(::boost::begin(rng), ::boost::end(rng), boost::container::new_allocator<void>());
+    }
+
+    /*!
+    \brief The constructor.
+
+    The tree is created using packing algorithm and a temporary packing allocator.
+
+    \param first             The beginning of the range of Values.
+    \param last              The end of the range of Values.
+    \param parameters        The parameters object.
+    \param getter            The function object extracting Indexable from Value.
+    \param equal             The function object comparing Values.
+    \param allocator         The allocator object for persistent data in the tree.
+    \param temp_allocator    The temporary allocator object used when packing.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Iterator, typename PackAlloc>
+    inline rtree(Iterator first, Iterator last,
+                 parameters_type const& parameters,
+                 indexable_getter const& getter,
+                 value_equal const& equal,
+                 allocator_type const& allocator,
+                 PackAlloc const& temp_allocator)
+        : m_members(getter, equal, parameters, allocator)
+    {
+        pack_construct(first, last, temp_allocator);
+    }
+
+    /*!
+    \brief The constructor.
+
+    The tree is created using packing algorithm and a temporary packing allocator.
+
+    \param rng               The range of Values.
+    \param parameters        The parameters object.
+    \param getter            The function object extracting Indexable from Value.
+    \param equal             The function object comparing Values.
+    \param allocator         The allocator object for persistent data in the tree.
+    \param temp_allocator    The temporary allocator object used when packing.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Range, typename PackAlloc>
+    inline explicit rtree(Range const& rng,
+                          parameters_type const& parameters,
+                          indexable_getter const& getter,
+                          value_equal const& equal,
+                          allocator_type const& allocator,
+                          PackAlloc const& temp_allocator)
+        : m_members(getter, equal, parameters, allocator)
+    {
+        pack_construct(::boost::begin(rng), ::boost::end(rng), temp_allocator);
+    }
+
+    /*!
+    \brief The constructor.
+
+    The tree is created using packing algorithm and a temporary packing allocator.
+
+    \param first        The beginning of the range of Values.
+    \param last         The end of the range of Values.
+    \param allocator    The allocator object for persistent data in the tree.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Iterator>
+    inline rtree(Iterator first, Iterator last,
+                 allocator_type const& allocator)
+        : m_members(indexable_getter(), value_equal(), parameters_type(), allocator)
+    {
+        pack_construct(first, last, boost::container::new_allocator<void>());
+    }
+
+    /*!
+    \brief The constructor.
+
+    The tree is created using packing algorithm and a temporary packing allocator.
+
+    \param rng          The range of Values.
+    \param allocator    The allocator object for persistent data in the tree.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Range>
+    inline explicit rtree(Range const& rng,
+                          allocator_type const& allocator)
+        : m_members(indexable_getter(), value_equal(), parameters_type(), allocator)
+    {
+        pack_construct(::boost::begin(rng), ::boost::end(rng), boost::container::new_allocator<void>());
+    }
+
+    /*!
+    \brief The constructor.
+
+    The tree is created using packing algorithm and a temporary packing allocator.
+
+    \param first             The beginning of the range of Values.
+    \param last              The end of the range of Values.
+    \param allocator         The allocator object for persistent data in the tree.
+    \param temp_allocator    The temporary allocator object used when packing.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Iterator, typename PackAlloc>
+    inline rtree(Iterator first, Iterator last,
+                 allocator_type const& allocator,
+                 PackAlloc const& temp_allocator)
+        : m_members(indexable_getter(), value_equal(), parameters_type(), allocator)
+    {
+        pack_construct(first, last, temp_allocator);
+    }
+
+    /*!
+    \brief The constructor.
+
+    The tree is created using packing algorithm and a temporary packing allocator.
+
+    \param rng               The range of Values.
+    \param allocator         The allocator object for persistent data in the tree.
+    \param temp_allocator    The temporary allocator object used when packing.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Range, typename PackAlloc>
+    inline explicit rtree(Range const& rng,
+                          allocator_type const& allocator,
+                          PackAlloc const& temp_allocator)
+        : m_members(indexable_getter(), value_equal(), parameters_type(), allocator)
+    {
+        pack_construct(::boost::begin(rng), ::boost::end(rng), temp_allocator);
     }
 
     /*!
@@ -462,7 +720,7 @@ public:
             // (allocators stored as base classes of members_holder)
             // copying them changes values_count, in this case it doesn't cause errors since data must be copied
             
-            typedef boost::mpl::bool_<
+            typedef std::integral_constant<bool,
                 allocator_traits_type::propagate_on_container_copy_assignment::value
             > propagate;
             
@@ -512,7 +770,7 @@ public:
                 // (allocators stored as base classes of members_holder)
                 // moving them changes values_count
                 
-                typedef boost::mpl::bool_<
+                typedef std::integral_constant<bool,
                     allocator_traits_type::propagate_on_container_move_assignment::value
                 > propagate;
                 detail::move_cond(this_allocs, src_allocs, propagate());
@@ -549,7 +807,7 @@ public:
         // (allocators stored as base classes of members_holder)
         // swapping them changes values_count
         
-        typedef boost::mpl::bool_<
+        typedef std::integral_constant<bool,
             allocator_traits_type::propagate_on_container_swap::value
         > propagate;
         detail::swap_cond(m_members.allocators(), other.m_members.allocators(), propagate());
@@ -629,10 +887,11 @@ public:
         if ( !m_members.root )
             this->raw_create();
 
-        typedef boost::mpl::bool_
-            <
-                boost::is_convertible<ConvertibleOrRange, value_type>::value
-            > is_conv_t;
+        typedef std::is_convertible<ConvertibleOrRange, value_type> is_conv_t;
+        typedef range::detail::is_range<ConvertibleOrRange> is_range_t;
+        BOOST_GEOMETRY_STATIC_ASSERT((is_conv_t::value || is_range_t::value),
+            "The argument has to be convertible to Value type or be a Range.",
+            ConvertibleOrRange);
 
         this->insert_dispatch(conv_or_rng, is_conv_t());
     }
@@ -728,10 +987,11 @@ public:
         if ( !m_members.root )
             return 0;
 
-        typedef boost::mpl::bool_
-            <
-                boost::is_convertible<ConvertibleOrRange, value_type>::value
-            > is_conv_t;
+        typedef std::is_convertible<ConvertibleOrRange, value_type> is_conv_t;
+        typedef range::detail::is_range<ConvertibleOrRange> is_range_t;
+        BOOST_GEOMETRY_STATIC_ASSERT((is_conv_t::value || is_range_t::value),
+            "The argument has to be convertible to Value type or be a Range.",
+            ConvertibleOrRange);
 
         return this->remove_dispatch(conv_or_rng, is_conv_t());
     }
@@ -827,14 +1087,9 @@ public:
     template <typename Predicates, typename OutIter>
     size_type query(Predicates const& predicates, OutIter out_it) const
     {
-        if ( !m_members.root )
-            return 0;
-
-        static const unsigned distance_predicates_count = detail::predicates_count_distance<Predicates>::value;
-        static const bool is_distance_predicate = 0 < distance_predicates_count;
-        BOOST_MPL_ASSERT_MSG((distance_predicates_count <= 1), PASS_ONLY_ONE_DISTANCE_PREDICATE, (Predicates));
-
-        return query_dispatch(predicates, out_it, boost::mpl::bool_<is_distance_predicate>());
+        return m_members.root
+             ? query_dispatch(predicates, out_it)
+             : 0;
     }
 
     /*!
@@ -928,7 +1183,18 @@ public:
         return const_query_iterator();
     }
 
-#ifndef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
+private:
+    template <typename Predicates>
+    using query_iterator_t = std::conditional_t
+        <
+            detail::predicates_count_distance<Predicates>::value == 0,
+            detail::rtree::iterators::spatial_query_iterator<members_holder, Predicates>,
+            detail::rtree::iterators::distance_query_iterator<members_holder, Predicates>
+        >;
+
+#ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL_ITERATORS
+public:
+#else
 private:
 #endif
     /*!
@@ -985,32 +1251,15 @@ private:
     \return             The iterator pointing at the begin of the query range.
     */
     template <typename Predicates>
-    typename boost::mpl::if_c<
-        detail::predicates_count_distance<Predicates>::value == 0,
-        detail::rtree::iterators::spatial_query_iterator<value_type, options_type, translator_type, box_type, allocators_type, Predicates>,
-        detail::rtree::iterators::distance_query_iterator<
-            value_type, options_type, translator_type, box_type, allocators_type, Predicates,
-            detail::predicates_find_distance<Predicates>::value
-        >
-    >::type
-    qbegin_(Predicates const& predicates) const
+    query_iterator_t<Predicates> qbegin_(Predicates const& predicates) const
     {
-        static const unsigned distance_predicates_count = detail::predicates_count_distance<Predicates>::value;
-        BOOST_MPL_ASSERT_MSG((distance_predicates_count <= 1), PASS_ONLY_ONE_DISTANCE_PREDICATE, (Predicates));
+        BOOST_GEOMETRY_STATIC_ASSERT((detail::predicates_count_distance<Predicates>::value <= 1),
+            "Only one distance predicate can be passed.",
+            Predicates);
 
-        typedef typename boost::mpl::if_c<
-            detail::predicates_count_distance<Predicates>::value == 0,
-            detail::rtree::iterators::spatial_query_iterator<value_type, options_type, translator_type, box_type, allocators_type, Predicates>,
-            detail::rtree::iterators::distance_query_iterator<
-                value_type, options_type, translator_type, box_type, allocators_type, Predicates,
-                detail::predicates_find_distance<Predicates>::value
-            >
-        >::type iterator_type;
-
-        if ( !m_members.root )
-            return iterator_type(m_members.translator(), predicates);
-
-        return iterator_type(m_members.root, m_members.translator(), predicates);
+        return m_members.root
+             ? query_iterator_t<Predicates>(m_members, predicates)
+             : query_iterator_t<Predicates>(predicates);
     }
 
     /*!
@@ -1046,29 +1295,13 @@ private:
     \return             The iterator pointing at the end of the query range.
     */
     template <typename Predicates>
-    typename boost::mpl::if_c<
-        detail::predicates_count_distance<Predicates>::value == 0,
-        detail::rtree::iterators::spatial_query_iterator<value_type, options_type, translator_type, box_type, allocators_type, Predicates>,
-        detail::rtree::iterators::distance_query_iterator<
-            value_type, options_type, translator_type, box_type, allocators_type, Predicates,
-            detail::predicates_find_distance<Predicates>::value
-        >
-    >::type
-    qend_(Predicates const& predicates) const
+    query_iterator_t<Predicates> qend_(Predicates const& predicates) const
     {
-        static const unsigned distance_predicates_count = detail::predicates_count_distance<Predicates>::value;
-        BOOST_MPL_ASSERT_MSG((distance_predicates_count <= 1), PASS_ONLY_ONE_DISTANCE_PREDICATE, (Predicates));
+        BOOST_GEOMETRY_STATIC_ASSERT((detail::predicates_count_distance<Predicates>::value <= 1),
+            "Only one distance predicate can be passed.",
+            Predicates);
 
-        typedef typename boost::mpl::if_c<
-            detail::predicates_count_distance<Predicates>::value == 0,
-            detail::rtree::iterators::spatial_query_iterator<value_type, options_type, translator_type, box_type, allocators_type, Predicates>,
-            detail::rtree::iterators::distance_query_iterator<
-                value_type, options_type, translator_type, box_type, allocators_type, Predicates,
-                detail::predicates_find_distance<Predicates>::value
-            >
-        >::type iterator_type;
-
-        return iterator_type(m_members.translator(), predicates);
+        return query_iterator_t<Predicates>(m_members.parameters(), m_members.translator(), predicates);
     }
 
     /*!
@@ -1169,10 +1402,9 @@ public:
     */
     const_iterator begin() const
     {
-        if ( !m_members.root )
-            return const_iterator();
-
-        return const_iterator(m_members.root);
+        return m_members.root
+             ? const_iterator(m_members.root)
+             : const_iterator();
     }
 
     /*!
@@ -1267,8 +1499,10 @@ public:
 
         if ( m_members.root )
         {
-            detail::rtree::visitors::children_box<value_type, options_type, translator_type, box_type, allocators_type>
-                box_v(result, m_members.translator());
+            detail::rtree::visitors::children_box
+                <
+                    members_holder
+                > box_v(result, m_members.parameters(), m_members.translator());
             detail::rtree::apply_visitor(box_v, *m_members.root);
         }
 
@@ -1295,32 +1529,18 @@ public:
             return 0;
 
         // the input should be convertible to Value or Indexable type
-
-        enum { as_val = 0, as_ind, dont_know };
-        typedef boost::mpl::int_
+        typedef typename index::detail::convertible_type
             <
-                boost::is_same<ValueOrIndexable, value_type>::value ?
-                    as_val :
-                    boost::is_same<ValueOrIndexable, indexable_type>::value ?
-                        as_ind :
-                        boost::is_convertible<ValueOrIndexable, indexable_type>::value ?
-                            as_ind :
-                            boost::is_convertible<ValueOrIndexable, value_type>::value ?
-                                as_val :
-                                dont_know
-            > variant;
-
-        BOOST_MPL_ASSERT_MSG((variant::value != dont_know),
-                             PASSED_OBJECT_NOT_CONVERTIBLE_TO_VALUE_NOR_INDEXABLE_TYPE,
-                             (ValueOrIndexable));
-
-        typedef typename boost::mpl::if_c
-            <
-                variant::value == as_val,
+                ValueOrIndexable,
                 value_type,
                 indexable_type
             >::type value_or_indexable;
 
+        static const bool is_void = std::is_void<value_or_indexable>::value;
+        BOOST_GEOMETRY_STATIC_ASSERT((! is_void),
+            "The argument has to be convertible to Value or Indexable type.",
+            ValueOrIndexable);
+
         // NOTE: If an object of convertible but not the same type is passed
         // into the function, here a temporary will be created.
         return this->template raw_count<value_or_indexable>(vori);
@@ -1444,12 +1664,9 @@ private:
         // CONSIDER: alternative - ignore invalid indexable or throw an exception
         BOOST_GEOMETRY_INDEX_ASSERT(detail::is_valid(m_members.translator()(value)), "Indexable is invalid");
 
-        detail::rtree::visitors::insert<
-            value_type,
-            value_type, options_type, translator_type, box_type, allocators_type,
-            typename options_type::insert_tag
-        > insert_v(m_members.root, m_members.leafs_level, value,
-                   m_members.parameters(), m_members.translator(), m_members.allocators());
+        detail::rtree::visitors::insert<value_type, members_holder>
+            insert_v(m_members.root, m_members.leafs_level, value,
+                     m_members.parameters(), m_members.translator(), m_members.allocators());
 
         detail::rtree::apply_visitor(insert_v, *m_members.root);
 
@@ -1475,10 +1692,9 @@ private:
         // TODO: awulkiew - assert for correct value (indexable) ?
         BOOST_GEOMETRY_INDEX_ASSERT(m_members.root, "The root must exist");
 
-        detail::rtree::visitors::remove<
-            value_type, options_type, translator_type, box_type, allocators_type
-        > remove_v(m_members.root, m_members.leafs_level, value,
-                   m_members.parameters(), m_members.translator(), m_members.allocators());
+        detail::rtree::visitors::remove<members_holder>
+            remove_v(m_members.root, m_members.leafs_level, value,
+                     m_members.parameters(), m_members.translator(), m_members.allocators());
 
         detail::rtree::apply_visitor(remove_v, *m_members.root);
 
@@ -1523,9 +1739,8 @@ private:
     {
         if ( t.m_members.root )
         {
-            detail::rtree::visitors::destroy<value_type, options_type, translator_type, box_type, allocators_type>
-                del_v(t.m_members.root, t.m_members.allocators());
-            detail::rtree::apply_visitor(del_v, *t.m_members.root);
+            detail::rtree::visitors::destroy<members_holder>
+                ::apply(t.m_members.root, t.m_members.allocators());
 
             t.m_members.root = 0;
         }
@@ -1546,8 +1761,7 @@ private:
     */
     inline void raw_copy(rtree const& src, rtree & dst, bool copy_tr_and_params) const
     {
-        detail::rtree::visitors::copy<value_type, options_type, translator_type, box_type, allocators_type>
-            copy_v(dst.m_members.allocators());
+        detail::rtree::visitors::copy<members_holder> copy_v(dst.m_members.allocators());
 
         if ( src.m_members.root )
             detail::rtree::apply_visitor(copy_v, *src.m_members.root);                      // MAY THROW (V, E: alloc, copy, N: alloc)
@@ -1562,9 +1776,9 @@ private:
         // TODO use subtree_destroyer
         if ( dst.m_members.root )
         {
-            detail::rtree::visitors::destroy<value_type, options_type, translator_type, box_type, allocators_type>
-                del_v(dst.m_members.root, dst.m_members.allocators());
-            detail::rtree::apply_visitor(del_v, *dst.m_members.root);
+            detail::rtree::visitors::destroy<members_holder>
+                ::apply(dst.m_members.root, dst.m_members.allocators());
+
             dst.m_members.root = 0;
         }
 
@@ -1583,7 +1797,7 @@ private:
     */
     template <typename ValueConvertible>
     inline void insert_dispatch(ValueConvertible const& val_conv,
-                                boost::mpl::bool_<true> const& /*is_convertible*/)
+                                std::true_type /*is_convertible*/)
     {
         this->raw_insert(val_conv);
     }
@@ -1598,12 +1812,8 @@ private:
     */
     template <typename Range>
     inline void insert_dispatch(Range const& rng,
-                                boost::mpl::bool_<false> const& /*is_convertible*/)
+                                std::false_type /*is_convertible*/)
     {
-        BOOST_MPL_ASSERT_MSG((detail::is_range<Range>::value),
-                             PASSED_OBJECT_IS_NOT_CONVERTIBLE_TO_VALUE_NOR_A_RANGE,
-                             (Range));
-
         typedef typename boost::range_const_iterator<Range>::type It;
         for ( It it = boost::const_begin(rng); it != boost::const_end(rng) ; ++it )
             this->raw_insert(*it);
@@ -1619,7 +1829,7 @@ private:
     */
     template <typename ValueConvertible>
     inline size_type remove_dispatch(ValueConvertible const& val_conv,
-                                     boost::mpl::bool_<true> const& /*is_convertible*/)
+                                     std::true_type /*is_convertible*/)
     {
         return this->raw_remove(val_conv);
     }
@@ -1634,12 +1844,8 @@ private:
     */
     template <typename Range>
     inline size_type remove_dispatch(Range const& rng,
-                                     boost::mpl::bool_<false> const& /*is_convertible*/)
+                                     std::false_type /*is_convertible*/)
     {
-        BOOST_MPL_ASSERT_MSG((detail::is_range<Range>::value),
-                             PASSED_OBJECT_IS_NOT_CONVERTIBLE_TO_VALUE_NOR_A_RANGE,
-                             (Range));
-
         size_type result = 0;
         typedef typename boost::range_const_iterator<Range>::type It;
         for ( It it = boost::const_begin(rng); it != boost::const_end(rng) ; ++it )
@@ -1653,15 +1859,16 @@ private:
     \par Exception-safety
     strong
     */
-    template <typename Predicates, typename OutIter>
-    size_type query_dispatch(Predicates const& predicates, OutIter out_it, boost::mpl::bool_<false> const& /*is_distance_predicate*/) const
+    template
+    <
+        typename Predicates, typename OutIter,
+        std::enable_if_t<(detail::predicates_count_distance<Predicates>::value == 0), int> = 0
+    >
+    size_type query_dispatch(Predicates const& predicates, OutIter out_it) const
     {
-        detail::rtree::visitors::spatial_query<value_type, options_type, translator_type, box_type, allocators_type, Predicates, OutIter>
-            find_v(m_members.translator(), predicates, out_it);
-
-        detail::rtree::apply_visitor(find_v, *m_members.root);
-
-        return find_v.found_count;
+        detail::rtree::visitors::spatial_query<members_holder, Predicates, OutIter>
+            query(m_members, predicates, out_it);
+        return query.apply(m_members);
     }
 
     /*!
@@ -1670,26 +1877,21 @@ private:
     \par Exception-safety
     strong
     */
-    template <typename Predicates, typename OutIter>
-    size_type query_dispatch(Predicates const& predicates, OutIter out_it, boost::mpl::bool_<true> const& /*is_distance_predicate*/) const
+    template
+    <
+        typename Predicates, typename OutIter,
+        std::enable_if_t<(detail::predicates_count_distance<Predicates>::value > 0), int> = 0
+    >
+    size_type query_dispatch(Predicates const& predicates, OutIter out_it) const
     {
-        BOOST_GEOMETRY_INDEX_ASSERT(m_members.root, "The root must exist");
+        BOOST_GEOMETRY_STATIC_ASSERT((detail::predicates_count_distance<Predicates>::value == 1),
+                                     "Only one distance predicate can be passed.",
+                                     Predicates);
 
-        static const unsigned distance_predicate_index = detail::predicates_find_distance<Predicates>::value;
-        detail::rtree::visitors::distance_query<
-            value_type,
-            options_type,
-            translator_type,
-            box_type,
-            allocators_type,
-            Predicates,
-            distance_predicate_index,
-            OutIter
-        > distance_v(m_members.parameters(), m_members.translator(), predicates, out_it);
+        detail::rtree::visitors::distance_query<members_holder, Predicates>
+            distance_v(m_members, predicates);
 
-        detail::rtree::apply_visitor(distance_v, *m_members.root);
-
-        return distance_v.finish();
+        return distance_v.apply(m_members, out_it);
     }
     
     /*!
@@ -1706,68 +1908,39 @@ private:
         detail::rtree::visitors::count
             <
                 ValueOrIndexable,
-                value_type,
-                options_type,
-                translator_type,
-                box_type,
-                allocators_type
-            > count_v(vori, m_members.translator());
+                members_holder
+            > count_v(vori, m_members.parameters(), m_members.translator());
 
         detail::rtree::apply_visitor(count_v, *m_members.root);
 
         return count_v.found_count;
     }
 
-    struct members_holder
-        : public translator_type
-        , public Parameters
-        , public allocators_type
+    /*!
+    \brief The constructor TODO.
+
+    The tree is created using packing algorithm.
+
+    \param first             The beginning of the range of Values.
+    \param last              The end of the range of Values.
+    \param temp_allocator    The temporary allocator object to be used by the packing algorithm.
+
+    \par Throws
+    \li If allocator copy constructor throws.
+    \li If Value copy constructor or copy assignment throws.
+    \li If allocation throws or returns invalid value.
+    */
+    template<typename Iterator, typename PackAlloc>
+    inline void pack_construct(Iterator first, Iterator last, PackAlloc const& temp_allocator)
     {
-    private:
-        members_holder(members_holder const&);
-        members_holder & operator=(members_holder const&);
-
-    public:
-        template <typename IndGet, typename ValEq, typename Alloc>
-        members_holder(IndGet const& ind_get,
-                       ValEq const& val_eq,
-                       Parameters const& parameters,
-                       BOOST_FWD_REF(Alloc) alloc)
-            : translator_type(ind_get, val_eq)
-            , Parameters(parameters)
-            , allocators_type(boost::forward<Alloc>(alloc))
-            , values_count(0)
-            , leafs_level(0)
-            , root(0)
-        {}
-
-        template <typename IndGet, typename ValEq>
-        members_holder(IndGet const& ind_get,
-                       ValEq const& val_eq,
-                       Parameters const& parameters)
-            : translator_type(ind_get, val_eq)
-            , Parameters(parameters)
-            , allocators_type()
-            , values_count(0)
-            , leafs_level(0)
-            , root(0)
-        {}
-
-        translator_type const& translator() const { return *this; }
-
-        IndexableGetter const& indexable_getter() const { return *this; }
-        IndexableGetter & indexable_getter() { return *this; }
-        EqualTo const& equal_to() const { return *this; }
-        EqualTo & equal_to() { return *this; }
-        Parameters const& parameters() const { return *this; }
-        Parameters & parameters() { return *this; }
-        allocators_type const& allocators() const { return *this; }
-        allocators_type & allocators() { return *this; }
-
-        size_type values_count;
-        size_type leafs_level;
-        node_pointer root;
-    };
+        typedef detail::rtree::pack<members_holder> pack;
+        size_type vc = 0, ll = 0;
+        m_members.root = pack::apply(first, last, vc, ll,
+                                     m_members.parameters(), m_members.translator(),
+                                     m_members.allocators(), temp_allocator);
+        m_members.values_count = vc;
+        m_members.leafs_level = ll;
+    }
 
     members_holder m_members;
 };
