@@ -2,6 +2,12 @@
 // Use, modification and distribution are subject to the Boost Software License,
 // Version 1.0. See http://www.boost.org/LICENSE_1_0.txt.
 
+#include "../test/test.hpp"
+
+#if defined(BOOST_CONVERT_IS_NOT_SUPPORTED)
+int main(int, char const* []) { return 0; }
+#else
+
 #include <boost/convert.hpp>
 #include <boost/convert/lexical_cast.hpp>
 
@@ -99,3 +105,5 @@ main(int, char const* [])
 
     return boost::report_errors();
 }
+
+#endif
