@@ -1,41 +1,38 @@
 var namespaceboost_1_1locale =
 [
-    [ "as", "namespaceboost_1_1locale_1_1as.html", null ],
     [ "boundary", "namespaceboost_1_1locale_1_1boundary.html", "namespaceboost_1_1locale_1_1boundary" ],
     [ "conv", "namespaceboost_1_1locale_1_1conv.html", "namespaceboost_1_1locale_1_1conv" ],
-    [ "flags", "namespaceboost_1_1locale_1_1flags.html", null ],
     [ "gnu_gettext", "namespaceboost_1_1locale_1_1gnu__gettext.html", "namespaceboost_1_1locale_1_1gnu__gettext" ],
     [ "period", "namespaceboost_1_1locale_1_1period.html", "namespaceboost_1_1locale_1_1period" ],
-    [ "time_zone", "namespaceboost_1_1locale_1_1time__zone.html", null ],
     [ "utf", "namespaceboost_1_1locale_1_1utf.html", "namespaceboost_1_1locale_1_1utf" ],
     [ "util", "namespaceboost_1_1locale_1_1util.html", "namespaceboost_1_1locale_1_1util" ],
-    [ "collator_base", "classboost_1_1locale_1_1collator__base.html", "classboost_1_1locale_1_1collator__base" ],
+    [ "abstract_calendar", "classboost_1_1locale_1_1abstract__calendar.html", "classboost_1_1locale_1_1abstract__calendar" ],
+    [ "basic_format", "classboost_1_1locale_1_1basic__format.html", "classboost_1_1locale_1_1basic__format" ],
+    [ "basic_message", "classboost_1_1locale_1_1basic__message.html", "classboost_1_1locale_1_1basic__message" ],
+    [ "calendar", "classboost_1_1locale_1_1calendar.html", "classboost_1_1locale_1_1calendar" ],
+    [ "calendar_facet", "classboost_1_1locale_1_1calendar__facet.html", "classboost_1_1locale_1_1calendar__facet" ],
     [ "collator", "classboost_1_1locale_1_1collator.html", "classboost_1_1locale_1_1collator" ],
+    [ "collator_base", "classboost_1_1locale_1_1collator__base.html", "classboost_1_1locale_1_1collator__base" ],
     [ "comparator", "structboost_1_1locale_1_1comparator.html", "structboost_1_1locale_1_1comparator" ],
-    [ "converter_base", "classboost_1_1locale_1_1converter__base.html", "classboost_1_1locale_1_1converter__base" ],
     [ "converter", "classboost_1_1locale_1_1converter.html", "classboost_1_1locale_1_1converter" ],
+    [ "converter_base", "classboost_1_1locale_1_1converter__base.html", "classboost_1_1locale_1_1converter__base" ],
+    [ "date_time", "classboost_1_1locale_1_1date__time.html", "classboost_1_1locale_1_1date__time" ],
+    [ "date_time_duration", "classboost_1_1locale_1_1date__time__duration.html", "classboost_1_1locale_1_1date__time__duration" ],
     [ "date_time_error", "classboost_1_1locale_1_1date__time__error.html", "classboost_1_1locale_1_1date__time__error" ],
     [ "date_time_period", "structboost_1_1locale_1_1date__time__period.html", "structboost_1_1locale_1_1date__time__period" ],
     [ "date_time_period_set", "classboost_1_1locale_1_1date__time__period__set.html", "classboost_1_1locale_1_1date__time__period__set" ],
-    [ "calendar", "classboost_1_1locale_1_1calendar.html", "classboost_1_1locale_1_1calendar" ],
-    [ "date_time", "classboost_1_1locale_1_1date__time.html", "classboost_1_1locale_1_1date__time" ],
-    [ "date_time_duration", "classboost_1_1locale_1_1date__time__duration.html", "classboost_1_1locale_1_1date__time__duration" ],
-    [ "posix_time", "structboost_1_1locale_1_1posix__time.html", "structboost_1_1locale_1_1posix__time" ],
-    [ "abstract_calendar", "classboost_1_1locale_1_1abstract__calendar.html", "classboost_1_1locale_1_1abstract__calendar" ],
-    [ "calendar_facet", "classboost_1_1locale_1_1calendar__facet.html", "classboost_1_1locale_1_1calendar__facet" ],
-    [ "basic_format", "classboost_1_1locale_1_1basic__format.html", "classboost_1_1locale_1_1basic__format" ],
-    [ "ios_info", "classboost_1_1locale_1_1ios__info.html", "classboost_1_1locale_1_1ios__info" ],
     [ "generator", "classboost_1_1locale_1_1generator.html", "classboost_1_1locale_1_1generator" ],
-    [ "generic_codecvt_base", "classboost_1_1locale_1_1generic__codecvt__base.html", "classboost_1_1locale_1_1generic__codecvt__base" ],
     [ "generic_codecvt", "classboost_1_1locale_1_1generic__codecvt.html", null ],
+    [ "generic_codecvt< CharType, CodecvtImpl, 1 >", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_011_01_4.html", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_011_01_4" ],
     [ "generic_codecvt< CharType, CodecvtImpl, 2 >", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_012_01_4.html", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_012_01_4" ],
     [ "generic_codecvt< CharType, CodecvtImpl, 4 >", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_014_01_4.html", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_014_01_4" ],
-    [ "generic_codecvt< CharType, CodecvtImpl, 1 >", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_011_01_4.html", "classboost_1_1locale_1_1generic__codecvt_3_01CharType_00_01CodecvtImpl_00_011_01_4" ],
+    [ "generic_codecvt_base", "classboost_1_1locale_1_1generic__codecvt__base.html", "classboost_1_1locale_1_1generic__codecvt__base" ],
     [ "hold_ptr", "classboost_1_1locale_1_1hold__ptr.html", "classboost_1_1locale_1_1hold__ptr" ],
     [ "info", "classboost_1_1locale_1_1info.html", "classboost_1_1locale_1_1info" ],
+    [ "ios_info", "classboost_1_1locale_1_1ios__info.html", "classboost_1_1locale_1_1ios__info" ],
     [ "localization_backend", "classboost_1_1locale_1_1localization__backend.html", "classboost_1_1locale_1_1localization__backend" ],
     [ "localization_backend_manager", "classboost_1_1locale_1_1localization__backend__manager.html", "classboost_1_1locale_1_1localization__backend__manager" ],
     [ "message_format", "classboost_1_1locale_1_1message__format.html", "classboost_1_1locale_1_1message__format" ],
-    [ "basic_message", "classboost_1_1locale_1_1basic__message.html", "classboost_1_1locale_1_1basic__message" ],
+    [ "posix_time", "structboost_1_1locale_1_1posix__time.html", "structboost_1_1locale_1_1posix__time" ],
     [ "utf8_codecvt", "classboost_1_1locale_1_1utf8__codecvt.html", "classboost_1_1locale_1_1utf8__codecvt" ]
 ];
\ No newline at end of file
